<?php
/**
 * Helper function for promocode funcnality
 *
 * @author  TeamWP @Potenza Global Solutions
 * @package cardealer-promocode
 */

/**
 * Check before post save if promocode already exist.
 */
add_filter( 'acf/validate_value/name=promo_code', 'acf_unique_value_field', 10, 4 );
if ( ! function_exists( 'acf_unique_value_field' ) ) {
	/**
	 * Acf unique value
	 *
	 * @param string $valid .
	 * @param string $value .
	 * @param string $field .
	 * @param string $input .
	 */
	function acf_unique_value_field( $valid, $value, $field, $input ) {

		if ( ! $valid || ( ! isset( $_POST['post_ID'] ) && ! isset( $_POST['post_id'] ) ) ) { // phpcs:ignore WordPress.Security.NonceVerification
			return $valid;
		}

		if ( isset( $_POST['post_ID'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification
			$post_id = intval( $_POST['post_ID'] ); // phpcs:ignore WordPress.Security.NonceVerification
		} else {
			$post_id = intval( $_POST['post_id'] ); // phpcs:ignore WordPress.Security.NonceVerification
		}

		if ( ! $post_id ) {
			return $valid;
		}

		$post_type  = get_post_type( $post_id );
		$field_name = $field['name'];

		$args = array(
			'post_type'    => $post_type,
			'post_status'  => 'publish, draft, trash',
			'post__not_in' => array( $post_id ),
			'meta_query'   => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				array(
					'key'   => $field_name,
					'value' => $value,
				),
			),
		);

		$query = new WP_Query( $args );

		if ( count( $query->posts ) ) {
			return 'Promo code already exist. Please enter a unique ' . $field['label'];
		}

		return true;
	}
}

/**
 * Check and redirect user to promocode offer page
 */
add_action( 'wp_ajax_validate_promocode', 'cardealer_validate_promocode' );
add_action( 'wp_ajax_nopriv_validate_promocode', 'cardealer_validate_promocode' );
if ( ! function_exists( 'cardealer_validate_promocode' ) ) {
	/**
	 * Validate promocode
	 */
	function cardealer_validate_promocode() {

		$result = array(
			'status' => 'error',
			'msg'    => 'Invalid promo code!',
		);

		$errors           = '';
		$promocode_status = 'disable';

		if ( ! isset( $_POST['promocode_nonce'] ) || ! wp_verify_nonce( sanitize_key( $_POST['promocode_nonce'] ), 'cdhl-promocode-form' ) ) {
			$errors = esc_html__( 'Sorry, your nonce did not verify. Refresh Page and try again.', 'cardealer-promocode' ) . "\r\n";

		} else {
			$promocode = ( isset( $_POST['promocode'] ) && ! empty( $_POST['promocode'] ) ) ? sanitize_text_field( wp_unslash( $_POST['promocode'] ) ) : '';
			if ( empty( $promocode ) ) {
				$errors = esc_html__( 'Please enter valid promo code', 'cardealer-promocode' ) . "\r\n";
			} else {
					$args  = array(
						'post_type'   => 'cars_promocodes',
						'post_status' => 'publish, draft, trash',
						'meta_query'  => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
							'relation' => 'AND',
							array(
								'key'   => 'promo_code',
								'value' => $promocode,
							),
							array(
								'key'   => 'status',
								'value' => 'enable',
							),
						),
					);
					$query = new WP_Query( $args );
					while ( $query->have_posts() ) :
						$query->the_post();
						$pid              = get_the_ID();
						$promo_code_page  = get_post_meta( $pid, 'promo_code_page', true );
						$promocode_is     = get_post_meta( $pid, 'promo_code', true );
						$promocode_status = get_post_meta( $pid, 'status', true );
						if ( 'custom' !== $promo_code_page ) {
							$reditect_url = $promo_code_page;
						} else {
							$reditect_url = get_post_meta( $pid, 'promo_code_url', true );

						}

						if ( $promocode === $promocode_is && 'enable' === $promocode_status ) {
							$result = array(
								'status' => 'success',
								'url'    => $reditect_url,
							);
							break;
						}
					endwhile;
					wp_reset_postdata();

			}
		}

		if ( '' !== $errors ) {
			$result = array(
				'status' => 'error',
				'msg'    => $errors,
			);
		}
		echo wp_json_encode( $result );
		exit();
	}
}

if ( ! function_exists( 'cdhl_validate_promocode_template' ) ) {
	/**
	 * Validate promocode template
	 */
	function cdhl_validate_promocode_template() {

		if ( is_page_template( 'templates/promocode.php' ) && isset( $_POST['promocode'] ) && ! empty( $_POST['promocode'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification
			$promocode        = sanitize_text_field( wp_unslash( $_POST['promocode'] ) ); // phpcs:ignore WordPress.Security.NonceVerification
			$promocode_status = 'disable';
			$result           = false;

			$args = array(
				'post_type'   => 'cars_promocodes',
				'post_status' => 'publish, draft, trash',
				'meta_query'  => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
					'relation' => 'AND',
					array(
						'key'   => 'promo_code',
						'value' => $promocode,
					),
					array(
						'key'   => 'status',
						'value' => 'enable',
					),
				),
			);

			$query = new WP_Query( $args );

			while ( $query->have_posts() ) :
				$query->the_post();
				$pid              = get_the_ID();
				$promocode_is     = get_post_meta( $pid, 'promo_code', true );
				$promocode_status = get_post_meta( $pid, 'status', true );
				if ( $promocode === $promocode_is && 'enable' === $promocode_status ) {
					$result = true;
					break;
				}
			endwhile;
			wp_reset_postdata();

			if ( ! isset( $_POST['promocode'] ) && empty( $_POST['promocode'] ) && ! $result ) { // phpcs:ignore WordPress.Security.NonceVerification
				wp_safe_redirect( site_url() );
				exit();
			}
		} elseif ( is_page_template( 'templates/promocode.php' ) && ! isset( $_POST['promocode'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification
			wp_safe_redirect( site_url() );
			exit();
		}
	}
	add_action( 'cardealer_head_before', 'cdhl_validate_promocode_template' );
}

if ( ! function_exists( 'cdhl_get_promocode_options' ) ) {
	/**
	 * Get promocode options
	 */
	function cdhl_get_promocode_options() {
		$list        = array();
		$my_args     = array(
			'post_type'      => 'page',
			'posts_per_page' => -1,
			'post_status'    => 'publish',
			'meta_query'     => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				array(
					'key'   => '_wp_page_template',
					'value' => 'templates/promocode.php',
				),
			),
		);
		$promo_pages = get_posts( $my_args );
		if ( $promo_pages ) {
			foreach ( $promo_pages as $promopage ) :
				setup_postdata( $promopage );
				$list[ get_the_permalink( $promopage->ID ) ] = $promopage->post_title;
			endforeach;
		}
		wp_reset_postdata();
		$list['custom'] = 'Custom';
		return $list;
	}
}

add_filter( 'acf/load_field/name=promo_code_page', 'acf_load_promocode_field_choices' );
if ( ! function_exists( 'acf_load_promocode_field_choices' ) ) {
	/**
	 * Load promocode field choices
	 *
	 * @param string $field .
	 */
	function acf_load_promocode_field_choices( $field ) {

		$field['choices'] = array();
		$field_post       = get_post( $field['ID'] );
		if ( is_object( $field_post ) && 'acf-field' === $field_post->post_type ) {
			return $field;
		}
		$field['choices'] = cdhl_get_promocode_options();
		return $field;
	}
}

if ( ! function_exists( 'cdhl_check_promocode_exist' ) ) {
	/**
	 * Function to check promocode available or not.
	 */
	function cdhl_check_promocode_exist() {
		$args  = array(
			'post_type'   => 'cars_promocodes',
			'post_status' => 'publish',
		);
		$query = new WP_Query( $args );
		if ( count( $query->posts ) ) {
			return true;
		}
		return false;
	}
}

/**
 * Function to add promocode button in cardealer panel.
 *
 * @param array $features features.
 */
function cardealer_promocode_extend_cardealer_other_fetures( $features ) {

	$features['promocode'] = array(
		'title' => esc_html__( 'Promocode', 'cardealer-promocode' ),
		'link'  => admin_url( 'edit.php?post_type=cars_promocodes' ),
		'desc'  => esc_html__( 'Click this button to open Promocode section.', 'cardealer-promocode' ),
	);

	return $features;
}
add_filter( 'cardealer_other_fetures', 'cardealer_promocode_extend_cardealer_other_fetures' );
